import { Controller } from '@hotwired/stimulus';

export default class extends Controller {
  static targets = ['input', 'fileName', 'fileInfoContainer', 'placeholder'];

  connect() {
    this.inputTarget.addEventListener('change', this.displayFileNames.bind(this));
  }

  disconnect() {
    this.inputTarget.removeEventListener('change', this.displayFileNames.bind(this));
  }

  selectFile() {
    this.inputTarget.click();
  }

  highlight(event) {
    event.preventDefault();
    this.element.classList.add('bg-gray-200');
  }

  dropFile(event) {
    event.preventDefault();
    this.element.classList.remove('bg-gray-200');
    const files = event.dataTransfer.files;
    this.inputTarget.files = files;
    this.displayFileNames();
    this.inputTarget.dispatchEvent(new Event('change'));
  }

  displayFileNames() {
    this.fileInfoContainerTarget.innerHTML = '';
    if (this.inputTarget.files.length > 0) {
      Array.from(this.inputTarget.files).forEach(file => {
        this.createFileInfoElement(this.fileInfoContainerTarget, file);
      });

      this.placeholderTarget.classList.add('hidden');
    } else {
      this.placeholderTarget.classList.remove('hidden');
    }
  }

  removeFile(event) {
    event.stopPropagation();
    const dataTransfer = new DataTransfer();
    const indexToRemove = Array.from(event.target.parentNode.parentNode.children).indexOf(event.target.parentNode);

    Array.from(this.inputTarget.files).forEach((file, index) => {
      if (index !== indexToRemove) {
        dataTransfer.items.add(file);
      }
    });

    this.inputTarget.files = dataTransfer.files;
    this.inputTarget.dispatchEvent(new Event('change'));
  }

  createFileInfoElement(container, file) {
    const fileInfoDiv = document.createElement('div');
    fileInfoDiv.className = 'flex items-center justify-between px-4 py-2 first:pt-4 last:pb-4';

    const fileNameSpan = document.createElement('span');
    fileNameSpan.className = 'text-gray-700';
    fileNameSpan.textContent += file.name;
    fileInfoDiv.appendChild(fileNameSpan);

    const removeButton = document.createElement('button');
    removeButton.type = 'button';
    removeButton.className = 'text-spirit-red hover:text-red-700';
    removeButton.dataset.action = 'click->file-upload#removeFile';
    removeButton.textContent = 'Supprimer';
    fileInfoDiv.appendChild(removeButton);

    container.appendChild(fileInfoDiv);
  }
}
